#include "amb2621.h"
#include "Pic16F1527_hardware.h"
#include <stdlib.h>
#include <stdint.h>
#include <xc.h>

#include "BLE_LED_Control.h"

#define MAX_CHANNELS    (uint8_t) 4
#define MAX_BRIGHTNESS  (uint8_t) 0xFF

static uint8_t channelValues[MAX_CHANNELS+1];

static bool AddChannelPacket(uint8_t *pChannelPacket, uint8_t channelNo);

void
BLE_LED_control_GetChannelValuesAsPacket(uint8_t *pBleData, uint8_t *length)
{
    uint8_t i;
    *length = 0;

    for (i = 0x00; i < 5; i++)
    {
        AddChannelPacket(&pBleData[i * 5], i);
        *length += 5;
    }
}

void
BLE_LED_control_BleToPWM(uint8_t *pBleData, uint8_t length)
{
    uint8_t currentPosition = 0;

    bool channel1Set = false;
    bool channel2Set = false;
    bool channel3Set = false;
    bool channel4Set = false;
    bool brightnessSet = false;

    uint8_t channel1ValueTemp = 0;
    uint8_t channel2ValueTemp = 0;
    uint8_t channel3ValueTemp = 0;
    uint8_t channel4ValueTemp = 0;
    uint8_t channelBrightnessTemp = 0;

    float channelBrightnessFactor;

    if ((length - currentPosition) < 26)
    {
        while(currentPosition < length)
        {
            /* parse packet after packet*/
            if ((4 == pBleData[currentPosition + CONTROL_FRAME_OFFSET_LENGTH])
                && (0x01 == pBleData[currentPosition + CONTROL_FRAME_OFFSET_OPCODE_LSB])
                && (0x00 == pBleData[currentPosition + CONTROL_FRAME_OFFSET_OPCODE_MSB]))
            {
                switch (pBleData[currentPosition + CONTROL_FRAME_OFFSET_CHANNELNO])
                {
                    case 0:
                    {
                        brightnessSet = true;
                        channelBrightnessTemp = pBleData[currentPosition + CONTROL_FRAME_OFFSET_CHANELVALUE];
                        if (channelBrightnessTemp > MAX_BRIGHTNESS)
                        {
                            channelBrightnessTemp = MAX_BRIGHTNESS;
                        }
                        break;
                    }
                    case 1:
                    {
                        channel1ValueTemp = pBleData[currentPosition + CONTROL_FRAME_OFFSET_CHANELVALUE];
                        channel1Set = (bool) (channel1ValueTemp != channelValues[1]);
                        break;
                    }
                    case 2:
                    {
                        channel2ValueTemp = pBleData[currentPosition + CONTROL_FRAME_OFFSET_CHANELVALUE];
                        channel2Set = (bool) (channel2ValueTemp != channelValues[2]);
                        break;
                    }
                    case 3:
                    {
                        channel3ValueTemp = pBleData[currentPosition + CONTROL_FRAME_OFFSET_CHANELVALUE];
                        channel3Set = (bool) (channel3ValueTemp != channelValues[3]);
                        break;
                    }
                    case 4:
                    {
                        channel4ValueTemp = pBleData[currentPosition + CONTROL_FRAME_OFFSET_CHANELVALUE];
                        channel4Set = (bool) (channel4ValueTemp != channelValues[4]);
                        break;
                    }
                    default:
                        break;
                }
                currentPosition += pBleData[currentPosition + CONTROL_FRAME_OFFSET_LENGTH] + 1;
            }
            else
            {
                /* packet not valid*/
                break;
            }
        }

        /* Uebergabe an PWM, CH5 dimmt alle Kanle zu X% */

        if (brightnessSet)
        {
            channelValues[0] = channelBrightnessTemp;

            channelBrightnessFactor = (float) ((float) channelValues[0] / (float) 255);

            if (channel1Set)
            {
                channelValues[1] = channel1ValueTemp;
            }
//            CCPR7 = (uint8_t)((channelValues[1] * (channelValues[0] * 100 / 255)) / 100);
            
            if (channel2Set)
            {
                channelValues[2] = channel2ValueTemp;
            }
//            CCPR8 = (uint8_t)((channelValues[2] * (channelValues[0] * 100 / 255)) / 100);

            if (channel3Set)
            {
                channelValues[3] = channel3ValueTemp;
            }
//            CCPR9 = (uint8_t)((channelValues[3]* (channelValues[0] * 100 / 255)) / 100);

            if (channel4Set)
            {
                channelValues[4] = channel4ValueTemp;
            }
//            CCPR10 = (uint8_t)((channelValues[4] * (channelValues[0] * 100 / 255)) / 100);
            Hardware_setChannelValues(&channelValues[0]);
            Hardware_UpdatePWM();
        }
    }
}

bool
BLE_LED_control_ChannelValid(uint8_t channelNo)
{
    return(bool) (channelNo <= MAX_CHANNELS);
}

void
BLE_LED_control_GetChannelValue(uint8_t *pValue, uint8_t channelNo)
{
    *pValue = channelValues[channelNo];
}

void BLE_LED_control_InitChannels()
{
    Hardware_getChannelValues(&channelValues[0]);
}

static bool
AddChannelPacket(uint8_t *pChannelPacket, uint8_t channelNo)
{
    bool ret = false;
    if (BLE_LED_control_ChannelValid(channelNo))
    {
        pChannelPacket[CONTROL_FRAME_OFFSET_LENGTH] = 0x04;
        pChannelPacket[CONTROL_FRAME_OFFSET_OPCODE_LSB] = 0x01;
        pChannelPacket[CONTROL_FRAME_OFFSET_OPCODE_MSB] = 0x00;
        pChannelPacket[CONTROL_FRAME_OFFSET_CHANNELNO] = channelNo;
        BLE_LED_control_GetChannelValue(&pChannelPacket[CONTROL_FRAME_OFFSET_CHANELVALUE], channelNo);

        ret = true;
    }

    return ret;
}

