/*
 * File:   LCD.c
 * Author: alexander.zeller
 *
 * Created on 16. Mai 2018, 08:36
 */

//Includes
#include <stdio.h>
#include <stdlib.h>
#include "LCD.h"
#include <stdint.h>
#include <xc.h>

/*
 * 
 */


// Globale Variablen
char shadowString[32]; // Globaler string fr Funktion putsLCD


char ch1 = 1;
char ch1_ = 1;
char ch2 = 0;
char ch2_ = 0;
char ch3 = 1;
char ch3_ = 1;
char ch4 = 0;
char ch4_ = 0;
char str[16];
char str2[16];

void
initLCD()
{
    // 15mS delay after Vdd reaches nnVdc before proceeding with LCD initialization
    // not always required and is based on system Vdd rise rate
    delay_ms(15);
    /* set initial states for the data and control pins */
    LATF &= 0xFF00;
    RW = 0; // R/W state set low
    RS = 0; // RS state set low
    E = 0; // E state set low

    /* set data and control pins to outputs */
    TRISF &= 0xFF00;
    RW_TRIS = 0; // RW pin set as output
    RS_TRIS = 0; // RS pin set as output
    E_TRIS = 0; // E pin set as output

    /* 1st LCD initialization sequence */
    DATA &= 0xFF00;
    DATA |= 0x0038;
    clockEnableLCD(); // toggle E signal
    delay_ms(5);

    /* 2nd LCD initialization sequence */
    DATA &= 0xFF00;
    DATA |= 0x0038;
    clockEnableLCD(); // toggle E signal
    delay_ms(1);

    /* 3rd LCD initialization sequence */
    DATA &= 0xFF00;
    DATA |= 0x0038;
    clockEnableLCD();
    delay_ms(1);
    sendCommandLCD(0x38); // function set
    delay_ms(5);
    sendCommandLCD(0x0C); // Display on/off control, cursor blink off (0x0C)
    delay_ms(5);
    sendCommandLCD(0x06); // entry mode set (0x06)
    delay_ms(5);
    sendCommandLCD(0x01); // clear display
    delay_ms(5);
}

/********************************************//**
 * 
 *   @author Alexander Zeller
 *   @brief - Senden eines Befehls an das LCD Display ( z.B. Clear )
 *   @attention - Blocking Code
 *    
 **********************************************/

void
sendCommandLCDblock(uint8_t data)
{

    DATA &= 0xFF00; // prepare RD0 - RD7
    DATA |= data; // command byte to lcd
    RW = 0; // ensure RW is 0
    RS = 0;
    clockEnableLCD();
    delay_ms(5);
}

/**********************************************
 * 
 *   @author Alexander Zeller
 *   @brief - Senden eines Befehls an das LCD Display ( z.B. Clear )
 *   @attention - Non Blocking Code
 *    
 **********************************************/
void
sendCommandLCD(uint8_t data)
{

    DATA &= 0xFF00; // prepare RD0 - RD7
    DATA |= data; // command byte to lcd
    RW = 0; // ensure RW is 0
    RS = 0;
    clockEnableLCD();
}

/**********************************************
 * 
 *   @author Alexander Zeller
 *   @brief - Schreiben eines Zeichens auf das LCD Display
 *   @attention - Blocking Code
 *    
 **********************************************/
void
writeDataLCDblock(uint8_t data)
{
    RW = 0; // ensure RW is 0
    RS = 1; // assert register select to 1
    DATA &= 0xFF00; // prepare RD0 - RD7
    DATA |= data; // data byte to lcd
    clockEnableLCD();
    RS = 0; // negate register select to 0

    delay_ms(1);
}

/**********************************************
 * 
 *   @author Alexander Zeller
 *   @brief - Schreiben eines Zeichens auf das LCD Display
 *   @attention - Non Blocking Code
 *    
 **********************************************/
void
writeDataLCD(uint8_t data)
{
    RW = 0; // ensure RW is 0
    RS = 1; // assert register select to 1
    DATA &= 0xFF00; // prepare RD0 - RD7
    DATA |= data; // data byte to lcd
    clockEnableLCD();
    RS = 0; // negate register select to 0     
}

/**********************************************
 * 
 *   @author Alexander Zeller
 *   @brief - Verzgerung um Milisekunden
 *   @param flag - abfrage des Busy Flags um nur zu schreiben wenn LCD nicht beschftigt
 * 
 **********************************************/

uint8_t
readBusyFlagLCD()
{
    /*  uint8_t flag =0;

      RS = 0;
      RW = 1;       
      ANSELF &= 0xFF;
      TRISF |= 0xFF;
    
      //clockEnableLCD();
    
     // if (PORTFbits.RF7 == 1 ) flag =1;
      flag = PORTFbits.RF7;
      RS = 0;
      RW = 0;
      ANSELF &= 0xFF;
      TRISF &= 0xFF;
     */
    return 0; //flag;

}

/**********************************************
 * 
 *   @author Alexander Zeller
 *   @brief - Verzgerung um Milisekunden
 *   @param flag - abfrage des Busy Flags um nur zu schreiben wenn LCD nicht beschftigt
 *          schadowString - Globaler Buffer um non blocking code zu erhalten
 *          x - Momentan zu schreibendes Zeichen aus dem Buffer
 *   @warning - Fr das schreiben in der Zweiten Zeile mssen die ersten 16 Zeichen gefllt sein 
 *   @attention - Funktioniert nur bei einem Aufrum momentan
 *    
 **********************************************/

void
putsLCD(char *buffer)
{
    //static uint8_t state = 0;
    uint8_t flag = readBusyFlagLCD();
    static uint8_t line = 0;
    if (flag == '1')
    {

    }
    else
    {
        int i = 0;
        static int x = 0;

        // Zeichenkette in Buffer schreiben
        for (i = 0; i < 32; i++)
        {
            if (*buffer != '\0')
                shadowString[i] = *buffer++;
            else
                shadowString[i] = ' ';

        }

        // Buffer an LCD senden
        if (shadowString[x] != '\0')
        {
            // Erste 16 Zeichen
            if (x < 16)
            {
                writeDataLCD(shadowString[x]);
                x++;
            }
            else if (x != 32) //zweite 16 Zeichen
            {
                if (line == 0)
                {
                    secondLine();
                    line = 1;
                }
                else
                {
                    writeDataLCD(shadowString[x]);
                    x++;
                }
            }
        }
        else
        {
            x = 0;
            firstLine();
            line = 0;
        }

    }
}

void
clockEnableLCD()
{
    E = 1;
    NOP();
    NOP();
    NOP();
    E = 0;
}

/**********************************************
 * 
 *   @author Alexander Zeller
 *   @brief - Verzgerung um Milisekunden
 *   @param u16milliseconds - angegebene Verzgerung in ms
 *    
 **********************************************/
void
delay_ms(unsigned int cycles)
{ //fr maximale frequenz
    unsigned int i;
    for (i = 1; i <= cycles; i++)
    {
        __delay_ms(1);
    }
}

/**********************************************
 * 
 *   @author Alexander Zeller
 *   @brief - Schreiben eines einzelnen Zeichens auf LCD
 *    
 **********************************************/
void
putcLCD(char c)
{
    uint8_t flag = readBusyFlagLCD();
    if (flag == '1')
    {

    }
    else
    {
        writeDataLCD((uint8_t) c);
    }
}

/**********************************************
 * 
 *   @author Alexander Zeller
 *   @brief - LCD Zeile 2  & Zeichen 1 Auswhlen
 *    
 **********************************************/
void
secondLine()
{
    sendCommandLCD(0xc0);
}

/**********************************************
 * 
 *   @author Alexander Zeller
 *   @brief - LCD Zeile 1 & Zeichen 1 Auswhlen
 *    
 **********************************************/
void
firstLine()
{
    sendCommandLCD(0x80);
}

//Lscht SchadowString

void
clearStrLCD()
{
    int i = 0;
    for (i = 0; i < 32; i++)
    {
        shadowString[i] = ' ';
    }

}

void
pwmToString()
{

    ch1 = CCPR7;
    ch1_ = (uint8_t) ((ch1 * 100) / 255);
    ch2 = CCPR8;
    ch2_ = (uint8_t) ((ch2 * 100) / 255);

    sprintf(str, "1:%u %%  2:%u %%", ch1_, ch2_);
    stringLCD(str, 1);

    ch3 = CCPR9;
    ch3_ = (uint8_t) ((ch3 * 100) / 255);
    ch4 = CCPR10;
    ch4_ = (uint8_t) ((ch4 * 100) / 255);

    sprintf(str2, "3:%u %%  4:%u %%", ch3_, ch4_);
    stringLCD(str2, 2);
}

//Teilt die verschiedenen Strings im ShadowString dem richtigen speicher zu

void
stringLCD(char *buffer, uint8_t line)
{
    int i = 0;
    if (line == 1)
        for (i = 0; i < 16; i++)
        {
            if (*buffer != '\0')
                shadowString[i] = *buffer++;
            else
                shadowString[i] = ' ';
        }
    else if (line == 2)
        for (i = 16; i < 32; i++)
        {
            if (*buffer != '\0')
                shadowString[i] = *buffer++;
            else
                shadowString[i] = ' ';
        }

}

//Schreibt aktuellen ShadowString ans LCD

void
putStrLCD()
{
    //static uint8_t state = 0;
    uint8_t flag = readBusyFlagLCD();
    static uint8_t line = 0;
    if (flag == '1')
    {

    }
    else
    {
        static int x = 0;
        // Buffer an LCD senden
        if (shadowString[x] != '\0' && x < 32)
        {
            // Erste 16 Zeichen
            if (x < 16)
            {
                writeDataLCD(shadowString[x]);
                x++;
            }
            else if (x != 32) //zweite 16 Zeichen
            {
                if (line == 0)
                {
                    secondLine();
                    line = 1;
                }
                else
                {
                    writeDataLCD(shadowString[x]);
                    x++;
                }
            }
        }
        else
        {
            x = 0;
            firstLine();
            line = 0;
        }

    }
}

void
putStrLCD2()
{
    //static uint8_t state = 0;
    uint8_t line = 0;
    int x;

    x = 0;
    firstLine();
    line = 0;

    while(x < 32)
    {
        // Buffer an LCD senden
        __delay_us(100);
        // Erste 16 Zeichen
        if (x < 16)
        {
            writeDataLCD(shadowString[x]);
            x++;
        }
        else if (line == 0)
        {
            secondLine();
            line = 1;
        }
        else//zweite 16 Zeichen
        {
            writeDataLCD(shadowString[x]);
            x++;
        }

    }
}