/*
 * File:   Pic16F1527_hardware.c
 * Author: alexander.zeller
 *
 * Created on 18. April 2018, 11:31
 */

#include "Pic16F1527_hardware.h"
#include <stdlib.h>
#include <stdint.h>
#include <xc.h>


static uint8_t channel1Value = 0;
static uint8_t channel2Value = 0;
static uint8_t channel3Value = 0;
static uint8_t channel4Value = 0;
static uint8_t channelBrightness = 0;

static int switch_adc_reading_flag = 0; /* Wenn wechsel zwischen Switch 
                        * und Poti  muss das Flag gecleared werden, 
                        * um neue ADC Werte fr den Switch einzulesen*/
unsigned static int8_t value = 0; //Wert zum Dimmen von allen PWM per ADC

void
OscConfig()
{
    // Internal OSC with 16MHZ
    OSCCON = 0x7A;
    // SBOREN disabled; BORFS disabled; 
    BORCON = 0x00;
}

void
WDT_Config(void)
{

    // WDTPS 1:65536; SWDTEN OFF; 
    WDTCON = 0x16;
}

void
PinConfig() // Muss vor den ADC ausgefhrt werden
{
    // Weak Pull up Disable 
    WPUB = 0x00;
    WPUD = 0x00;
    WPUE = 0x00;
    WPUG = 0x00;



    // set Pins to low
    LATA = 0x00;
    LATB = 0x00;
    LATC = 0x00;
    LATD = 0x00;
    LATE = 0x00;
    LATF = 0x00;
    LATG = 0x00;

    // Setting Pinoptions for UserPanel
    TRISAbits.TRISA0 = 1;
    ANSELAbits.ANSA0 = 0;
    TRISD = 0xFF; // Register D Input
    ANSELD = 0x00; // Register D Digital
    TRISEbits.TRISE7 = 1; //input
    TRISEbits.TRISE6 = 1; //input
    TRISBbits.TRISB0 = 1; //input
    ANSELBbits.ANSB0 = 0; //Digital
    TRISBbits.TRISB1 = 1; //input
    ANSELBbits.ANSB1 = 1; // Analog
    TRISBbits.TRISB2 = 1; //input
    ANSELBbits.ANSB2 = 1; // Analog
    TRISBbits.TRISB3 = 1; //input
    ANSELBbits.ANSB3 = 1; // Analog
    TRISBbits.TRISB4 = 1; //input
    ANSELBbits.ANSB4 = 1; // Analog
    TRISBbits.TRISB5 = 1; //input
    ANSELBbits.ANSB5 = 1; // Analog

    // Setting Pinoptions for LCD
    ANSELF = 0x00;
    //TODO

}

void
initADC_AN18()
{
    ADCON0 = 0x49; // AN18
    ADCON1 = 0x60; // Left,fosc/64 and vref = vdd
}

void
initADC_AN19()
{
    ADCON0 = 0x4D; // AN19
    ADCON1 = 0x60; // Left,fosc/64 and vref = vdd
}

void
initADC_AN20()
{
    ADCON0 = 0x51; // AN20
    ADCON1 = 0x60; // Left,fosc/64 and vref = vdd
}

void
initADC_AN21()
{
    ADCON0 = 0x55; // AN21
    ADCON1 = 0x60; // Left,fosc/64 and vref = vdd
}

void
initADC_AN22()
{
    ADCON0 = 0x59; // AN22
    ADCON1 = 0x60; // Left,fosc/64 and vref = vdd
}

void
initPWM()
{
    // Setting Pinoptions for PWM
    TRISEbits.TRISE2 = 1;
    TRISEbits.TRISE3 = 1;
    TRISEbits.TRISE4 = 1;
    TRISEbits.TRISE5 = 1;


    PR2 = 0xF9; // Load PWM Period value. Formula: p.185 datasheet 
    //-> 0xF9 fr 250 HZ 0xFF fr 244HZ

    /* Load the CCPxCON Registern and first bits of Duty Cycle */
    CCP7CON = 0b00001111;
    CCP8CON = 0b00001111;
    CCP9CON = 0b00001111;
    CCP10CON = 0b00001111;


    /* Load Duty Cycle init bits */
    CCPR7L = 0x7D;
    CCPR8L = 0x7D;
    CCPR9L = 0x7D;
    CCPR10L = 0x7D;


    /* PWM abhaengig von Timer 2 */
    CCPTMRS2 = 0x00;
    CCPTMRS1 = 0x00;

    /* Timer 2 Options/Configuration */
    PIR1 = 0; //Clear T2 IF
    T2CON = 0b0000111; // 1:1 Postscale, Timer ON, 1:64 Prescale
    /* Loop for Init */
    while((PIR1 & 0x02) == 0)
    {
    };

    /* Setting Pinoptions for PWM */
    ANSELEbits.ANSE2 = 0; // Needed for ccp10 
    TRISEbits.TRISE2 = 0; //ccp10 output
    TRISEbits.TRISE3 = 0; //ccp9 output
    TRISEbits.TRISE4 = 0; //ccp8 output
    TRISEbits.TRISE5 = 0; //ccp7 output


}

/* ADC Read Funcitons 
 * each ADC needs 2 cycles, one for the initiation and one for reading
 * all ADC got their own function
 */
unsigned int
read_ADC18()
{
    static int flag18 = 0;
    static uint8_t AN18_value = 0; // Wert fr AD Wandler

    if (flag18 == 0)
    {
        initADC_AN18();
        flag18 = 1;
    }
    else if (flag18 == 1)
    {
        ADCON0bits.GO_nDONE = 1; // Start Conversion
        while(ADCON0bits.GO_nDONE == 1)
        {
        }; //Wait for conversion to be done
        AN18_value = ADRESH;
        ADIF = 0;
        flag18 = 0;
    }
    return AN18_value;
}

unsigned int
read_ADC19()
{
    static int flag19 = 0;
    static uint8_t AN19_value = 0; // Wert fr AD Wandler

    if (flag19 == 0)
    {
        initADC_AN19();
        flag19 = 1;
    }
    else if (flag19 == 1)
    {
        ADCON0bits.GO_nDONE = 1; // Start Conversion
        while(ADCON0bits.GO_nDONE == 1)
        {
        }; //Wait for conversion to be done
        AN19_value = ADRESH;
        ADIF = 0;
        flag19 = 0;
    }
    return AN19_value;
}

unsigned int
read_ADC20()
{
    static int flag20 = 0;
    static uint8_t AN20_value = 0; // Wert fr AD Wandler

    if (flag20 == 0)
    {
        initADC_AN20();
        flag20 = 1;
    }
    else if (flag20 == 1)
    {
        ADCON0bits.GO_nDONE = 1; // Start Conversion
        while(ADCON0bits.GO_nDONE == 1)
        {
        }; //Wait for conversion to be done
        AN20_value = ADRESH;
        ADIF = 0;
        flag20 = 0;
    }
    return AN20_value;
}

unsigned int
read_ADC21()
{
    static int flag21 = 0;
    static uint8_t AN21_value = 0; // Wert fr AD Wandler

    if (flag21 == 0)
    {
        initADC_AN21();
        flag21 = 1;
    }
    else if (flag21 == 1)
    {
        ADCON0bits.GO_nDONE = 1; // Start Conversion
        while(ADCON0bits.GO_nDONE == 1)
        {
        }; //Wait for conversion to be done
        AN21_value = ADRESH;
        ADIF = 0;
        flag21 = 0;
    }
    return AN21_value;
}

unsigned int
read_ADC22()
{
    static int flag22 = 0;
    static uint8_t AN22_value = 0; // Wert fr AD Wandler

    if (flag22 == 0)
    {
        initADC_AN22();
        flag22 = 1;
    }
    else if (flag22 == 1)
    {
        ADCON0bits.GO_nDONE = 1; // Start Conversion
        while(ADCON0bits.GO_nDONE == 1)
        {
        }; //Wait for conversion to be done
        AN22_value = ADRESH;
        ADIF = 0;
        flag22 = 0;
    }
    return AN22_value;
}

/* Start all ADC one by one !! needs 10 cycles to be done
 * 
 * Flag increases every 2nd cycle 
 *  
Funktion mit einzeldimmung und gesammtdimmung ueber Poti */

void
ADCtoPWM()
{

    static int ADChannel = AN18;
    static int flag = 0;
    switch_adc_reading_flag = 0;


    switch (ADChannel)
    {
        case AN18:
            /* Multipliziert den Wert des jeweiligem ADC mit dem 
             * Prozentsatz von ADC5 fr Gesammtdimmung*/

            channel4Value = read_ADC18();
            CCPR10L = (uint8_t)((channel4Value * (uint8_t) (channelBrightness * 100 / 255)) / 100);
            if (flag == 1)
            {
                ADChannel = AN19;
                initADC_AN19();
            }
            else
                flag++;

            break;

        case AN19:

            channel2Value = read_ADC19();
            CCPR8L = (uint8_t)((channel2Value * (uint8_t) (channelBrightness * 100 / 255)) / 100);

            if (flag == 2)
            {
                ADChannel = AN20;
                initADC_AN20();
            }
            else
                flag++;

            break;

        case AN20:

            channel1Value = read_ADC20();
            CCPR7L = (uint8_t)((channel1Value * (uint8_t) (channelBrightness * 100 / 255)) / 100);

            if (flag == 3)
            {
                ADChannel = AN21;
                initADC_AN21();
            }
            else
                flag++;

            break;


        case AN21:

            channel3Value = read_ADC21();
            CCPR9L = (uint8_t)((channel3Value * (uint8_t) (channelBrightness * 100 / 255)) / 100);

            if (flag == 4)
            {
                ADChannel = AN22;
                initADC_AN22();
            }
            else
                flag++;

            break;

        case AN22:

            channelBrightness = read_ADC22();
            if (flag >= 5)
            {
                ADChannel = AN18;
                flag = 0;
                initADC_AN18();
            }
            else
                flag++;

            break;

    }
}

/* Funktion mit einzeldimmung und gesammtdimmung ueber taster */
void
SwitchToPWM()
{
    static int counter = 0;


    /* getting start values from ADC when switching from ADC -> TODO 
     * at the moment initialisation with all channels =0 and ch5 
     * Value = 255 for max brightness
     * because for now we have nothing to check which brightness we
     * got when starting 
     */
    //    if (switch_adc_reading_flag == 0)
    //    {
    //        channel1Value = 0;
    //        channel2Value = 0;
    //        channel3Value = 0;
    //        channel4Value = 0;
    //        channelBrightness = 255;
    //
    //        switch_adc_reading_flag = 1;
    //
    //    }
    //
    //    else
    {
        //Taster alle 10 ms lesen und + oder -
        if (counter >= 9)
        {
            if (isPressed(SW9) == LOW)
            {
                if (channelBrightness > 0)
                    channelBrightness--;
            }
            else if (isPressed(SW8) == LOW)
            {
                if (channelBrightness < 255)
                    channelBrightness++;
            }

            if (isPressed(SW1) == LOW)
            {
                if (channel1Value > 0)
                    channel1Value--;
            }
            else if (isPressed(SW0) == LOW)
            {
                if (channel1Value < 255)
                    channel1Value++;
            }


            if (isPressed(SW3) == LOW)
            {
                if (channel2Value > 0)
                    channel2Value--;
            }
            else if (isPressed(SW2) == LOW)
            {

                if (channel2Value < 255)
                    channel2Value++;
            }

            if (isPressed(SW5) == LOW)
            {
                if (channel3Value > 0)
                    channel3Value--;
            }
            else if (isPressed(SW4) == LOW)
            {
                if (channel3Value < 255)
                    channel3Value++;
            }


            if (isPressed(SW7) == LOW)
            {
                if (channel4Value > 0)
                    channel4Value--;
            }
            else if (isPressed(SW6) == LOW)
            {
                if (channel4Value < 255)
                    channel4Value++;
            }



            counter = 0;
        }
        else counter++;
        /* Uebergabe an PWM, CH5 dimmt alle Kanle zu X% */
        Hardware_UpdatePWM();
    }
}

/* Lesen des Wertes der an dem jeweiligen Taster anliegt */
int
digitalRead(int port)
{
    int ret = 0;
    switch (port)
    {
        case SW0: //Ch1+
            ret = (PORTDbits.RD4);
            break;
        case SW1: //CH1-
            ret = (PORTDbits.RD2);
            break;
        case SW2: //CH2+
            ret = (PORTDbits.RD6);
            break;
        case SW3: //Ch2-
            ret = (PORTDbits.RD0);
            break;
        case SW4: //Ch3+
            ret = (PORTEbits.RE6);
            break;
        case SW5: //CH3-
            ret = (PORTDbits.RD1);
            break;
        case SW6: //Ch4+
            ret = (PORTDbits.RD7);
            break;
        case SW7: //Ch4-
            ret = (PORTDbits.RD3);
            break;
        case SW8: //Ch5+
            ret = (PORTEbits.RE7);
            break;
        case SW9: //Ch5-
            ret = (PORTDbits.RD5);
            break;
        case SW10: //Switch
            ret = (PORTBbits.RB0);
            break;
    }

    return ret;
}

/* Funktion fuer Auswahl ob Poti oder Taster - einzige Funktion die in  
 * Main aufgerufen werden muss */
void
SwitchOrPWM()
{
    if (isPressed(SW10) == HIGH)
        ADCtoPWM();
    else
        SwitchToPWM();
}

/* Cycle Time Funtions */

volatile uint8_t acessFlag = 0;
volatile uint8_t collisionFlag = 0;
volatile uint32_t interrupt_counter;

/* Setting 1ms Timer 1 */
void
setSystemTimer(void)
{

    T1CONbits.TMR1ON = 0b01; //Disable Timer
    T1CONbits.TMR1CS = 0b01; //Select internal instruction cycle clock 
    T1CONbits.SOSCEN = 0; //Disable 2nd Oscilator
    T1CONbits.T1CKPS = 0b00; //Select 1:1 Prescaler
    PIE1bits.TMR1IE = 1;
    INTCONbits.PEIE = 1;
    INTCONbits.GIE = 1;

    TMR1 = 0x00; // Clear timer register
    TMR1H = 0xC1;
    TMR1L = 0x7F; // 0xC17F fr 49535 -> bei 16MHZ CPU Frequenz, 
    //Timer berlauf nach 1ms

    PIR1bits.TMR1IF = 0; // Clear interrupt Flag TMR1
    T1CONbits.TMR1ON = 1; // ENable Timer
    interrupt_counter = 0;


}

/* ISR Timer1 for 1ms Cycle Time, checking if collision @ reading time */
void
t1Int(void)
{
    interrupt_counter++;
    TMR1H = 0xC1; //Reload Timer Values
    TMR1L = 0x7F;
    if (acessFlag == 1)
    {
        collisionFlag = 1;
    }
    else
    {
        collisionFlag = 0;
    }
    PIR1bits.TMR1IF = 0; // Clear interrupt Flag TMR1
}

/* Counts Cycle time in 1ms steps */
unsigned int
getSystemTimeMillis(void)
{
    uint16_t sysTime;
    static uint16_t old_time;

    acessFlag = 1;
    sysTime = interrupt_counter;
    acessFlag = 0;

    if (collisionFlag == 1)
    {
        sysTime = old_time;
        collisionFlag = 0;
    }
    old_time = sysTime;
    return sysTime;

}

/* Tasterentprellung */
int
isPressed(int taster)
{
    static uint8_t zustand0 = HIGH_STABIL;
    static uint8_t zustand1 = HIGH_STABIL;
    static uint8_t zustand2 = HIGH_STABIL;
    static uint8_t zustand3 = HIGH_STABIL;
    static uint8_t zustand4 = HIGH_STABIL;
    static uint8_t zustand5 = HIGH_STABIL;
    static uint8_t zustand6 = HIGH_STABIL;
    static uint8_t zustand7 = HIGH_STABIL;
    static uint8_t zustand8 = HIGH_STABIL;
    static uint8_t zustand9 = HIGH_STABIL;
    static uint8_t zustand10 = HIGH_STABIL;
    static int ret = 0;

    switch (taster)
    {

        case SW0:
            switch (zustand0)
            {
                case HIGH_STABIL:
                    if (digitalRead(SW0) == 1)
                        zustand0 = HIGH_STABIL;
                    else
                        zustand0 = HIGH_INSTABIL;

                    ret = HIGH;
                    break;
                case HIGH_INSTABIL:
                    if (digitalRead(SW0) == 1)
                    {
                        zustand0 = HIGH_STABIL;
                        ret = HIGH;
                    }
                    else
                    {
                        zustand0 = LOW_STABIL;
                        ret = LOW;
                    }
                    break;
                case LOW_STABIL:
                    if (digitalRead(SW0) == 0)
                    {
                        zustand0 = LOW_STABIL;
                    }
                    else
                    {
                        zustand0 = LOW_INSTABIL;
                    }

                    ret = LOW;
                    break;
                case LOW_INSTABIL:
                    if (digitalRead(SW0) == 0)
                    {
                        zustand0 = LOW_STABIL;
                        ret = LOW;
                    }
                    else
                    {
                        zustand0 = HIGH_STABIL;
                        ret = HIGH;
                    }
                    break;
            }
            break;
        case SW1:
            switch (zustand1)
            {
                case HIGH_STABIL:
                    if (digitalRead(SW1) == 1)
                        zustand1 = HIGH_STABIL;
                    else
                        zustand1 = HIGH_INSTABIL;

                    ret = HIGH;
                    break;
                case HIGH_INSTABIL:
                    if (digitalRead(SW1) == 1)
                    {
                        zustand1 = HIGH_STABIL;
                        ret = HIGH;
                    }
                    else
                    {
                        zustand1 = LOW_STABIL;
                        ret = LOW;
                    }
                    break;
                case LOW_STABIL:
                    if (digitalRead(SW1) == 0)
                    {
                        zustand1 = LOW_STABIL;
                    }
                    else
                    {
                        zustand1 = LOW_INSTABIL;
                    }

                    ret = LOW;
                    break;
                case LOW_INSTABIL:
                    if (digitalRead(SW1) == 0)
                    {
                        zustand1 = LOW_STABIL;
                        ret = LOW;
                    }
                    else
                    {
                        zustand1 = HIGH_STABIL;
                        ret = HIGH;
                    }
                    break;
            }
            break;

        case SW2:
            switch (zustand2)
            {
                case HIGH_STABIL:
                    if (digitalRead(SW2) == 1)
                        zustand2 = HIGH_STABIL;
                    else
                        zustand2 = HIGH_INSTABIL;

                    ret = HIGH;
                    break;
                case HIGH_INSTABIL:
                    if (digitalRead(SW2) == 1)
                    {
                        zustand2 = HIGH_STABIL;
                        ret = HIGH;
                    }
                    else
                    {
                        zustand2 = LOW_STABIL;
                        ret = LOW;
                    }
                    break;
                case LOW_STABIL:
                    if (digitalRead(SW2) == 0)
                    {
                        zustand2 = LOW_STABIL;
                    }
                    else
                    {
                        zustand2 = LOW_INSTABIL;
                    }

                    ret = LOW;
                    break;
                case LOW_INSTABIL:
                    if (digitalRead(SW2) == 0)
                    {
                        zustand2 = LOW_STABIL;
                        ret = LOW;
                    }
                    else
                    {
                        zustand2 = HIGH_STABIL;
                        ret = HIGH;
                    }
                    break;
            }
            break;

        case SW3:
            switch (zustand3)
            {
                case HIGH_STABIL:
                    if (digitalRead(SW3) == 1)
                        zustand3 = HIGH_STABIL;
                    else
                        zustand3 = HIGH_INSTABIL;

                    ret = HIGH;
                    break;
                case HIGH_INSTABIL:
                    if (digitalRead(SW3) == 1)
                    {
                        zustand3 = HIGH_STABIL;
                        ret = HIGH;
                    }
                    else
                    {
                        zustand3 = LOW_STABIL;
                        ret = LOW;
                    }
                    break;
                case LOW_STABIL:
                    if (digitalRead(SW3) == 0)
                    {
                        zustand3 = LOW_STABIL;
                    }
                    else
                    {
                        zustand3 = LOW_INSTABIL;
                    }

                    ret = LOW;
                    break;
                case LOW_INSTABIL:
                    if (digitalRead(SW3) == 0)
                    {
                        zustand3 = LOW_STABIL;
                        ret = LOW;
                    }
                    else
                    {
                        zustand3 = HIGH_STABIL;
                        ret = HIGH;
                    }
                    break;
            }
            break;

        case SW4:
            switch (zustand4)
            {
                case HIGH_STABIL:
                    if (digitalRead(SW4) == 1)
                        zustand4 = HIGH_STABIL;
                    else
                        zustand4 = HIGH_INSTABIL;

                    ret = HIGH;
                    break;
                case HIGH_INSTABIL:
                    if (digitalRead(SW4) == 1)
                    {
                        zustand4 = HIGH_STABIL;
                        ret = HIGH;
                    }
                    else
                    {
                        zustand4 = LOW_STABIL;
                        ret = LOW;
                    }
                    break;
                case LOW_STABIL:
                    if (digitalRead(SW4) == 0)
                    {
                        zustand4 = LOW_STABIL;
                    }
                    else
                    {
                        zustand4 = LOW_INSTABIL;
                    }

                    ret = LOW;
                    break;
                case LOW_INSTABIL:
                    if (digitalRead(SW4) == 0)
                    {
                        zustand4 = LOW_STABIL;
                        ret = LOW;
                    }
                    else
                    {
                        zustand4 = HIGH_STABIL;
                        ret = HIGH;
                    }
                    break;
            }
            break;

        case SW5:
            switch (zustand5)
            {
                case HIGH_STABIL:
                    if (digitalRead(SW5) == 1)
                        zustand5 = HIGH_STABIL;
                    else
                        zustand5 = HIGH_INSTABIL;

                    ret = HIGH;
                    break;
                case HIGH_INSTABIL:
                    if (digitalRead(SW5) == 1)
                    {
                        zustand5 = HIGH_STABIL;
                        ret = HIGH;
                    }
                    else
                    {
                        zustand5 = LOW_STABIL;
                        ret = LOW;
                    }
                    break;
                case LOW_STABIL:
                    if (digitalRead(SW5) == 0)
                    {
                        zustand5 = LOW_STABIL;
                    }
                    else
                    {
                        zustand5 = LOW_INSTABIL;
                    }

                    ret = LOW;
                    break;
                case LOW_INSTABIL:
                    if (digitalRead(SW5) == 0)
                    {
                        zustand5 = LOW_STABIL;
                        ret = LOW;
                    }
                    else
                    {
                        zustand5 = HIGH_STABIL;
                        ret = HIGH;
                    }
                    break;
            }
            break;

        case SW6:
            switch (zustand6)
            {
                case HIGH_STABIL:
                    if (digitalRead(SW6) == 1)
                        zustand6 = HIGH_STABIL;
                    else
                        zustand6 = HIGH_INSTABIL;

                    ret = HIGH;
                    break;
                case HIGH_INSTABIL:
                    if (digitalRead(SW6) == 1)
                    {
                        zustand6 = HIGH_STABIL;
                        ret = HIGH;
                    }
                    else
                    {
                        zustand6 = LOW_STABIL;
                        ret = LOW;
                    }
                    break;
                case LOW_STABIL:
                    if (digitalRead(SW6) == 0)
                    {
                        zustand6 = LOW_STABIL;
                    }
                    else
                    {
                        zustand6 = LOW_INSTABIL;
                    }

                    ret = LOW;
                    break;
                case LOW_INSTABIL:
                    if (digitalRead(SW6) == 0)
                    {
                        zustand6 = LOW_STABIL;
                        ret = LOW;
                    }
                    else
                    {
                        zustand6 = HIGH_STABIL;
                        ret = HIGH;
                    }
                    break;
            }
            break;

        case SW7:
            switch (zustand7)
            {
                case HIGH_STABIL:
                    if (digitalRead(SW7) == 1)
                        zustand7 = HIGH_STABIL;
                    else
                        zustand7 = HIGH_INSTABIL;

                    ret = HIGH;
                    break;
                case HIGH_INSTABIL:
                    if (digitalRead(SW7) == 1)
                    {
                        zustand7 = HIGH_STABIL;
                        ret = HIGH;
                    }
                    else
                    {
                        zustand7 = LOW_STABIL;
                        ret = LOW;
                    }
                    break;
                case LOW_STABIL:
                    if (digitalRead(SW7) == 0)
                    {
                        zustand7 = LOW_STABIL;
                    }
                    else
                    {
                        zustand7 = LOW_INSTABIL;
                    }

                    ret = LOW;
                    break;
                case LOW_INSTABIL:
                    if (digitalRead(SW7) == 0)
                    {
                        zustand7 = LOW_STABIL;
                        ret = LOW;
                    }
                    else
                    {
                        zustand7 = HIGH_STABIL;
                        ret = HIGH;
                    }
                    break;
            }
            break;

        case SW8:
            switch (zustand8)
            {
                case HIGH_STABIL:
                    if (digitalRead(SW8) == 1)
                        zustand8 = HIGH_STABIL;
                    else
                        zustand8 = HIGH_INSTABIL;

                    ret = HIGH;
                    break;
                case HIGH_INSTABIL:
                    if (digitalRead(SW8) == 1)
                    {
                        zustand8 = HIGH_STABIL;
                        ret = HIGH;
                    }
                    else
                    {
                        zustand8 = LOW_STABIL;
                        ret = LOW;
                    }
                    break;
                case LOW_STABIL:
                    if (digitalRead(SW8) == 0)
                    {
                        zustand8 = LOW_STABIL;
                    }
                    else
                    {
                        zustand8 = LOW_INSTABIL;
                    }

                    ret = LOW;
                    break;
                case LOW_INSTABIL:
                    if (digitalRead(SW8) == 0)
                    {
                        zustand8 = LOW_STABIL;
                        ret = LOW;
                    }
                    else
                    {
                        zustand8 = HIGH_STABIL;
                        ret = HIGH;
                    }
                    break;
            }
            break;

        case SW9:
            switch (zustand9)
            {
                case HIGH_STABIL:
                    if (digitalRead(SW9) == 1)
                        zustand9 = HIGH_STABIL;
                    else
                        zustand9 = HIGH_INSTABIL;

                    ret = HIGH;
                    break;
                case HIGH_INSTABIL:
                    if (digitalRead(SW9) == 1)
                    {
                        zustand9 = HIGH_STABIL;
                        ret = HIGH;
                    }
                    else
                    {
                        zustand9 = LOW_STABIL;
                        ret = LOW;
                    }
                    break;
                case LOW_STABIL:
                    if (digitalRead(SW9) == 0)
                    {
                        zustand9 = LOW_STABIL;
                    }
                    else
                    {
                        zustand9 = LOW_INSTABIL;
                    }

                    ret = LOW;
                    break;
                case LOW_INSTABIL:
                    if (digitalRead(SW9) == 0)
                    {
                        zustand9 = LOW_STABIL;
                        ret = LOW;
                    }
                    else
                    {
                        zustand9 = HIGH_STABIL;
                        ret = HIGH;
                    }
                    break;
            }
            break;

        case SW10:
            switch (zustand10)
            {
                case HIGH_STABIL:
                    if (digitalRead(SW10) == 1)
                        zustand10 = HIGH_STABIL;
                    else
                        zustand10 = HIGH_INSTABIL;

                    ret = HIGH;
                    break;
                case HIGH_INSTABIL:
                    if (digitalRead(SW10) == 1)
                    {
                        zustand10 = HIGH_STABIL;
                        ret = HIGH;
                    }
                    else
                    {
                        zustand10 = LOW_STABIL;
                        ret = LOW;
                    }
                    break;
                case LOW_STABIL:
                    if (digitalRead(SW10) == 0)
                    {
                        zustand10 = LOW_STABIL;
                    }
                    else
                    {
                        zustand10 = LOW_INSTABIL;
                    }

                    ret = LOW;
                    break;
                case LOW_INSTABIL:
                    if (digitalRead(SW10) == 0)
                    {
                        zustand10 = LOW_STABIL;
                        ret = LOW;
                    }
                    else
                    {
                        zustand10 = HIGH_STABIL;
                        ret = HIGH;
                    }
                    break;
            }
            break;
    }
    return ret;
}

void
Hardware_getChannelValues(uint8_t *channelValues)
{
    channelValues[0] = channelBrightness;
    channelValues[1] = channel1Value;
    channelValues[2] = channel1Value;
    channelValues[3] = channel1Value;
    channelValues[4] = channel1Value;
}

void
Hardware_setChannelValues(uint8_t *channelValues)
{
    channelBrightness = channelValues[0];
    channel1Value = channelValues[1];
    channel2Value = channelValues[2];
    channel3Value = channelValues[3];
    channel4Value = channelValues[4];
}

void
Hardware_UpdatePWM()
{
    CCPR7 = (uint8_t) ((channel1Value * (channelBrightness * 100 / 255)) / 100);
    CCPR8 = (uint8_t) ((channel2Value * (channelBrightness * 100 / 255)) / 100);
    CCPR9 = (uint8_t) ((channel3Value * (channelBrightness * 100 / 255)) / 100);
    CCPR10 = (uint8_t) ((channel4Value * (channelBrightness * 100 / 255)) / 100);
}
/* Nicht mehr benoetigte Funktionen */

/* Funktion nur fuer einzeldimmung ueber taster */
/* void SwitchToPWM_einzel() 
{
    
    static unsigned int8_t CH1_value =0;
    static unsigned int8_t CH2_value =0;
    static unsigned int8_t CH3_value =0;
    static unsigned int8_t CH4_value =0;
    static unsigned int8_t CH5_value =0;
    static int counter =0;

    
    // getting start values from ADC when switching from ADC 
    // mus evtl noch ein delay eingefgt werden
    if (switch_adc_reading_flag ==0)
    {
        CH1_value = CCPR10L; // AN18_value; //read_ADC18();
        CH2_value = AN19_value;//CCPR9L; //read_ADC19();
        CH3_value = AN20_value;//CCPR8L;// read_ADC20();
        CH4_value = AN21_value;// CCPR7L; //read_ADC21();
        CH5_value = value; //read_ADC22();
        
        switch_adc_reading_flag = 1;
        
    }
   
    else
    {
        //Taster lesen und + oder -
        if (counter>=9)
        {     
            if ( isPressed (SW8)== LOW)
            {    
                if (CH5_value < 255)
                CH5_value ++;
            }
            else if (isPressed(SW9)== LOW)
            {   
                if (CH5_value > 0)
                CH5_value --;
            }

            if ( isPressed (SW0)== LOW)
            {    
                if (CH1_value < 255)
                CH1_value++; //=(((CH1_value+1)* (CH5_value*100/255) ) /100);
            }
            else if (isPressed(SW1)== LOW)
            {   
                if (CH1_value > 0)
               CH1_value--;// = (((CH1_value-1)* (CH5_value*100/255) ) /100);     
            }    
            CCPR7= CH1_value;

            if ( isPressed (SW2)== LOW)
            {    
                if (CH2_value < 255)
                CH2_value++;// = (((CH2_value+1)* (CH5_value*100/255) ) /100);
            }
            else if (isPressed(SW3)== LOW)
            {   
                if (CH2_value > 0)
                CH2_value--;// = (((CH2_value-1)* (CH5_value*100/255) ) /100);
            }
             CCPR8= CH2_value;

            if ( isPressed (SW4)== LOW)
            {    
                if (CH3_value < 255)
                CH3_value++;// = (((CH3_value+1)* (CH5_value*100/255) ) /100);
            }    
            else if (isPressed(SW5)== LOW)
            {   
                if (CH3_value > 0)
                CH3_value--;// = (((CH3_value-1)* (CH5_value*100/255) ) /100);   
            }
             CCPR9= CH3_value;

            if ( isPressed (SW6)== LOW)
            {    
                if (CH4_value < 255)
                CH4_value++;// = (((CH4_value+1)* (CH5_value*100/255) ) /100);
            }
            else if (isPressed(SW7)== LOW)
            {   
                if (CH4_value > 0)
                CH4_value--;// = (((CH4_value-1)* (CH5_value*100/255) ) /100);
            }
             CCPR10= CH4_value;
             
             
             counter=0;
        }
        else counter ++;
    }       
}
 */

/* Funktion nur fuer einzeldimmung ueber Poti 
void ADCtoPWM_einzel()
{   
    
    static int ADChannel = AN18;
    static int flag =0;
    switch_adc_reading_flag =0;
    
           
        switch (ADChannel)
        {
            case AN18: 
            CCPR10L = read_ADC18(); 
            if (flag == 1 )  
            {   
                ADChannel = AN19;
                initADC_AN19();
            }
            else
            flag++;
            
            break;
            
            case AN19:              

            CCPR8L = read_ADC19();
            
            if (flag == 2) 
            {  
                ADChannel = AN20;
                initADC_AN20();
            }
            else
            flag++;
                          
            break;
         
            case AN20:              
  
            CCPR7L = read_ADC20();
            
            if (flag == 3)  
            {  
                ADChannel = AN21;
                initADC_AN21();
            }
            else
            flag++;
                          
            break;            
            
                     
            case AN21:              

            CCPR9L = read_ADC21();
            
            if (flag == 4)  
            {  
                ADChannel = AN22;
                initADC_AN22();
            }
            else
            flag++;
                          
            break;      
                 
            case AN22:              

            CCPR6L = read_ADC22();
            
            if (flag >= 5)  
            {  
                ADChannel = AN18;
                flag =0;
                initADC_AN18();
            }
            else
            flag++;
                          
            break;      
            
        }
}
 */

/*
// delay x number of millseconds
void mydelay_ms(unsigned int cycles) {
    unsigned int i;

    for (i = 1; i <= cycles; i++) {
        __delay_ms(1);
    }
}

// delay x number of seconds
void mydelay_sec(unsigned int cycles) {
    unsigned char i;

    for (i = 1; i <= cycles; i++) {
        mydelay_ms(1000);
    }
}
 * */
