/**
 * \file header.h
 * \brief
 *
 * \since
 * \author
 */


#ifndef _AMB2621_H_
#define _AMB2621_H_


/* ###########################################################################
 * Includes:
 */

#include <stdint.h>
#include <stdbool.h>

/*
 * Includes.
 * ######################################################################### */

/* ###########################################################################
 * Macros & Typedefs:
 */

/* #############################
 * Constant Macros:
 */
#define CMD_STX 0x02

#define AMB2621_CMD_TYPE_REQ (uint8_t)(0 << 6)
#define AMB2621_CMD_TYPE_CNF (uint8_t)(1 << 6)
#define AMB2621_CMD_TYPE_IND (uint8_t)(2 << 6)
#define AMB8826_CMD_TYPE_RSP (uint8_t)(3 << 6)


#define AMB2621_CMD_RESET (uint8_t)0x00
#define AMB2621_CMD_RESET_REQ (AMB2621_CMD_RESET | AMB2621_CMD_TYPE_REQ)
#define AMB2621_CMD_RESET_CNF (AMB2621_CMD_RESET | AMB2621_CMD_TYPE_CNF)
#define AMB2621_CMD_RESET_IND (AMB2621_CMD_RESET | AMB2621_CMD_TYPE_IND)

#define AMB2621_CMD_GETSTATE (uint8_t)0x01
#define AMB2621_CMD_GETSTATE_REQ (AMB2621_CMD_GETSTATE | AMB2621_CMD_TYPE_REQ)
#define AMB2621_CMD_GETSTATE_CNF (AMB2621_CMD_GETSTATE | AMB2621_CMD_TYPE_CNF)
#define AMB2621_CMD_GETSTATE_IND (AMB2621_CMD_GETSTATE | AMB2621_CMD_TYPE_IND)

#define AMB2621_CMD_SLEEP (uint8_t)0x02
#define AMB2621_CMD_SLEEP_REQ (AMB2621_CMD_SLEEP | AMB2621_CMD_TYPE_REQ)
#define AMB2621_CMD_SLEEP_CNF (AMB2621_CMD_SLEEP | AMB2621_CMD_TYPE_CNF)
#define AMB2621_CMD_SLEEP_IND (AMB2621_CMD_SLEEP | AMB2621_CMD_TYPE_IND)

#define AMB2621_CMD_DATA (uint8_t)0x04
#define AMB2621_CMD_DATA_REQ (AMB2621_CMD_DATA | AMB2621_CMD_TYPE_REQ)
#define AMB2621_CMD_DATA_CNF (AMB2621_CMD_DATA | AMB2621_CMD_TYPE_CNF)
#define AMB2621_CMD_DATA_IND (AMB2621_CMD_DATA | AMB2621_CMD_TYPE_IND)

#define AMB2621_CMD_CONNECT (uint8_t)0x06
#define AMB2621_CMD_CONNECT_REQ (AMB2621_CMD_CONNECT | AMB2621_CMD_TYPE_REQ)
#define AMB2621_CMD_CONNECT_CNF (AMB2621_CMD_CONNECT | AMB2621_CMD_TYPE_CNF)
#define AMB2621_CMD_CONNECT_IND (AMB2621_CMD_CONNECT | AMB2621_CMD_TYPE_IND)
#define AMB2621_CMD_CHANNELOPEN_RSP (AMB2621_CMD_CONNECT | AMB8826_CMD_TYPE_RSP)

#define AMB2621_CMD_DISCONNECT (uint8_t)0x07
#define AMB2621_CMD_DISCONNECT_REQ (AMB2621_CMD_DISCONNECT | AMB2621_CMD_TYPE_REQ)
#define AMB2621_CMD_DISCONNECT_CNF (AMB2621_CMD_DISCONNECT | AMB2621_CMD_TYPE_CNF)
#define AMB2621_CMD_DISCONNECT_IND (AMB2621_CMD_DISCONNECT | AMB2621_CMD_TYPE_IND)

#define AMB2621_CMD_GET (uint8_t)0x10
#define AMB2621_CMD_GET_REQ (AMB2621_CMD_GET | AMB2621_CMD_TYPE_REQ)
#define AMB2621_CMD_GET_CNF (AMB2621_CMD_GET | AMB2621_CMD_TYPE_CNF)

#define AMB2621_CMD_SET (uint8_t)0x11
#define AMB2621_CMD_SET_REQ (AMB2621_CMD_SET | AMB2621_CMD_TYPE_REQ)
#define AMB2621_CMD_SET_CNF (AMB2621_CMD_SET | AMB2621_CMD_TYPE_CNF)

#define AMB2621_CMD_FACTORYRESET (uint8_t)0x1C
#define AMB2621_CMD_FACTORYRESET_REQ (AMB2621_CMD_FACTORYRESET | AMB2621_CMD_TYPE_REQ)
#define AMB2621_CMD_FACTORYRESET_CNF (AMB2621_CMD_FACTORYRESET | AMB2621_CMD_TYPE_CNF)

#define SEC_MODE_BONDING_ENABLE_MASK (uint8_t)0x04

/*
 * Constant Macros.
 * ########################### */

/* #############################
 * Function Macros:
 */

/*
 * Function Macros.
 * ########################## */

/* #############################
 * Typedefs:
 */

typedef enum AMB2621_States_t {
    State_Module_Reset =        (uint8_t)0x00,
    State_BLE_Channel_Open =    (uint8_t)0x01,
    State_BLE_Disconnected =    (uint8_t)0x02,
    State_BLE_Wait_For_Data =   (uint8_t)0x03,
    State_BLE_Data_Rcvd =       (uint8_t)0x04,
    State_Uart_Open =           (uint8_t)0x05,
} AMB2621_States_t;

typedef enum AMB2621_BLE_Role_t {
    AMB2621_BLE_Role_None =         (uint8_t)0x00,
    AMB2621_BLE_Role_Pheripheral =  (uint8_t)0x01,
    AMB2621_BLE_Role_Central =      (uint8_t)0x02,
    AMB2621_BLE_Role_DTM =          (uint8_t)0x10,
} AMB2621_BLE_Role_t;

typedef enum AMB2621_BLE_Action_t {
    AMB2621_BLE_Action_None =       (uint8_t)0x00,
    AMB2621_BLE_Action_Idle =       (uint8_t)0x01,
    AMB2621_BLE_Action_Scanning =   (uint8_t)0x02,
    AMB2621_BLE_Action_Connected =  (uint8_t)0x03,
    AMB2621_BLE_Action_Sleep =      (uint8_t)0x04,
    AMB2621_BLE_Action_DTM =        (uint8_t)0x05,
} AMB2621_BLE_Action_t;

typedef enum AMB2621_UserSettings_t {
    AMB2621_USERSETTING_POSITION_FS_FWVersion =             (uint8_t)0x01,
    AMB2621_USERSETTING_POSITION_RF_DEVICE_NAME =           (uint8_t)0x02,
    AMB2621_USERSETTING_POSITION_FS_MAC =                   (uint8_t)0x03,
    AMB2621_USERSETTING_POSITION_FS_BTMAC =                 (uint8_t)0x04,
    AMB2621_USERSETTING_POSITION_RF_ADVERTISING_TIMEOUT =   (uint8_t)0x07,
    AMB2621_USERSETTING_POSITION_RF_CONNECTION_TIMING =     (uint8_t)0x08,
    AMB2621_USERSETTING_POSITION_RF_SCAN_TIMING =           (uint8_t)0x09,
    AMB2621_USERSETTING_POSITION_RF_SCAN_FACTOR =           (uint8_t)0x0A,
    AMB2621_USERSETTING_POSITION_UART_BAUDRATE_INDEX =      (uint8_t)0x0B,
    AMB2621_USERSETTING_POSITION_RF_SEC_FLAGS =             (uint8_t)0x0C,
    AMB2621_USERSETTING_POSITION_RF_SCAN_FLAGS =            (uint8_t)0x0D,
    AMB2621_USERSETTING_POSITION_RF_BEACON_FLAGS =          (uint8_t)0x0E,
    AMB2621_USERSETTING_POSITION_FS_DEVICE_INFO =           (uint8_t)0x0F,
    AMB2621_USERSETTING_POSITION_FS_SERIAL_NUMBER =         (uint8_t)0x10,
    AMB2621_USERSETTING_POSITION_RF_TX_POWER =              (uint8_t)0x11,
    AMB2621_USERSETTING_POSITION_RF_STATIC_PASSKEY =        (uint8_t)0x12,
} AMB2621_UserSettings_t;

typedef enum AMB2621_SecMode_t {
    AMB2621_SecFalgs_NONE =                     (uint8_t)0x00,
    AMB2621_SecFalgs_JustWorks =                (uint8_t)0x02,
    AMB2621_SecFalgs_StaticPassKey =            (uint8_t)0x03,
    AMB2621_SecFlags_JustWorks_Bonding =        (uint8_t)(AMB2621_SecFalgs_JustWorks | SEC_MODE_BONDING_ENABLE_MASK),
    AMB2621_SecFalgs_StaticPassKey_Bonding =    (uint8_t)(AMB2621_SecFalgs_StaticPassKey | SEC_MODE_BONDING_ENABLE_MASK),
} AMB2621_SecMode_t;

typedef enum AMB2621_ConnectionTiming_t {
    AMB2621_ConnectionTiming_0 = (uint8_t)0x00,
    AMB2621_ConnectionTiming_1 = (uint8_t)0x01,
    AMB2621_ConnectionTiming_2 = (uint8_t)0x02,
    AMB2621_ConnectionTiming_3 = (uint8_t)0x03,
    AMB2621_ConnectionTiming_4 = (uint8_t)0x04,
    AMB2621_ConnectionTiming_5 = (uint8_t)0x05,
    AMB2621_ConnectionTiming_6 = (uint8_t)0x06

} AMB2621_ConnectionTiming_t;

typedef enum AMB2621_ScanTiming_t {
    AMB2621_ScanTiming_0 = (uint8_t)0x00,
    AMB2621_ScanTiming_1 = (uint8_t)0x01,
    AMB2621_ScanTiming_2 = (uint8_t)0x02,
    AMB2621_ScanTiming_3 = (uint8_t)0x03,
    AMB2621_ScanTiming_4 = (uint8_t)0x04,
    AMB2621_ScanTiming_5 = (uint8_t)0x05

} AMB2621_ScanTiming_t;

typedef enum AMB2621_TXPower_t {
    AMB2621_TXPower_4       = (int8_t) 4,
    AMB2621_TXPower_0       = (int8_t) 0,
    AMB2621_TXPower_minus4  = (int8_t)-4,
    AMB2621_TXPower_minus8  = (int8_t)-8,
    AMB2621_TXPower_minus12 = (int8_t)-12,
} AMB2621_TXPower_t;

typedef enum AMB2621_BaudRate_t {
    AMB2621_BaudRate_9600 =     (uint8_t)0x00,
    AMB2621_BaudRate_19200 =    (uint8_t)0x01,
    AMB2621_BaudRate_38400 =    (uint8_t)0x02,
    AMB2621_BaudRate_115200 =   (uint8_t)0x03,
    AMB2621_BaudRate_230400 =   (uint8_t)0x04
} AMB2621_BaudRate_t;
/*
 * Typedefs.
 * ########################## */

/*
 * Macros & Typedefs.
 * ######################################################################## */

/* ##########################################################################
 * Declarations:
 */

/* #############################
 * Exported Variables:
 */

AMB2621_States_t ble_state;

/*
 * Exported Variables.
 * ########################## */

/* #############################
 * Exported Functions:
 */
extern bool AMB2621_Open();
extern bool AMB2621_PinReset();
extern bool AMB2621_PinWakeup();
extern bool AMB2621_Reset();
extern bool AMB2621_Disconnect();
extern bool AMB2621_Sleep();

extern bool AMB2621_Transmit(uint8_t* pPayload, uint16_t length);
extern bool AMB2621_TransmitChannelValues(uint8_t* pPayload, uint8_t length);
extern bool AMB2621_InterpretUartRx();

extern void AMB2621_GetBLEMaxPayloadLength(uint8_t *maxPayload);
extern bool AMB2621_GetBLEData(uint8_t *pBuffer, uint8_t *pLength);
extern bool AMB2621_WaitForStartupIndication(uint16_t max_time_ms);

extern bool AMB2621_FactoryReset();
extern bool AMB2621_Set(uint8_t userSetting, uint8_t *pValue, uint8_t length);
extern bool AMB2621_SetDeviceName(uint8_t *deviceName, uint8_t nameLength);
extern bool AMB2621_SetAdvertisingTimeout(uint16_t advTimeout);
extern bool AMB2621_SetConnectionTiming(uint8_t connectionTiming);
extern bool AMB2621_SetSecFlag(AMB2621_SecMode_t secMode);
extern bool AMB2621_SetBaudrateIndex(AMB2621_BaudRate_t baudrateIndex);
extern bool AMB2621_SetStaticPasskey(uint8_t *staticPasskey);

extern bool AMB2621_Get(uint8_t userSetting, uint8_t *pResponse, uint8_t *pResponse_Length);
extern bool AMB2621_GetDeviceName(uint8_t *deviceName, uint8_t *nameLength);
extern bool AMB2621_GetMAC(uint8_t *pMac);
extern bool AMB2621_GetBTMAC(uint8_t *pMac);
extern bool AMB2621_GetAdvertisingTimeout(uint8_t *advTimeout);
extern bool AMB2621_GetConnectionTiming(uint8_t *connectionTiming);
extern bool AMB2621_GetSecFlag(uint8_t *psecMode);
extern bool AMB2621_GetBaudrateIndex(uint8_t *baudrateIndex);
extern bool AMB2621_GetStaticPasskey(uint8_t *staticPasskey);
extern bool AMB2621_GetState(AMB2621_BLE_Role_t *pBLE_role, AMB2621_BLE_Action_t *pBLE_action, uint8_t *pInfo, uint8_t *pLength);

extern void Uart_Timer_Callback();
/*
 * Exported Functions.
 * ########################## */

/*
 * Declarations.
 * ######################################################################### */

#endif /* #ifndef _AMB2621_H_ */