/**
  EUSART1 Generated Driver File

  @Company
    Microchip Technology Inc.

  @File Name
    eusart1.c

  @Summary
    This is the generated driver implementation file for the EUSART1 driver using PIC10 / PIC12 / PIC16 / PIC18 MCUs

  @Description
    This source file provides APIs for EUSART1.
    Generation Information :
        Product Revision  :  PIC10 / PIC12 / PIC16 / PIC18 MCUs - 1.65.2
        Device            :  PIC16F1527
        Driver Version    :  2.01
    The generated drivers are tested against the following:
        Compiler          :  XC8 1.45
        MPLAB 	          :  MPLAB X 4.15
 */

/*
    (c) 2018 Microchip Technology Inc. and its subsidiaries. 
    
    Subject to your compliance with these terms, you may use Microchip software and any 
    derivatives exclusively with Microchip products. It is your responsibility to comply with third party 
    license terms applicable to your use of third party software (including open source software) that 
    may accompany Microchip software.
    
    THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES, WHETHER 
    EXPRESS, IMPLIED OR STATUTORY, APPLY TO THIS SOFTWARE, INCLUDING ANY 
    IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS 
    FOR A PARTICULAR PURPOSE.
    
    IN NO EVENT WILL MICROCHIP BE LIABLE FOR ANY INDIRECT, SPECIAL, PUNITIVE, 
    INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, COST OR EXPENSE OF ANY KIND 
    WHATSOEVER RELATED TO THE SOFTWARE, HOWEVER CAUSED, EVEN IF MICROCHIP 
    HAS BEEN ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE FORESEEABLE. TO 
    THE FULLEST EXTENT ALLOWED BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL 
    CLAIMS IN ANY WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT 
    OF FEES, IF ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS 
    SOFTWARE.
 */

/**
  Section: Included Files
 */
#include "eusart1.h"

/**
  Section: Macro Declarations
 */

#define EUSART1_TX_BUFFER_SIZE 48
#define EUSART1_RX_BUFFER_SIZE 48

/**
  Section: Global Variables
 */
volatile uint8_t eusart1TxHead = 0;
volatile uint8_t eusart1TxTail = 0;
volatile uint8_t eusart1TxBuffer[EUSART1_TX_BUFFER_SIZE];
volatile uint8_t eusart1TxBufferRemaining;

volatile uint8_t eusart1RxHead = 0;
volatile uint8_t eusart1RxTail = 0;
volatile uint8_t eusart1RxBuffer[EUSART1_RX_BUFFER_SIZE];
volatile uint8_t eusart1RxCount;
volatile uint8_t rc1_error_state;

static void initUartTimer(uint8_t baudrateIndex);

/**
  Section: EUSART1 APIs
 */
void
EUSART1_Initialize(uint8_t baudrateIndex)
{
    // disable interrupts before changing states
    PIE1bits.RC1IE = 0;
    EUSART1_SetRxInterruptHandler(EUSART1_Receive_ISR);
    PIE1bits.TX1IE = 0;
    EUSART1_SetTxInterruptHandler(EUSART1_Transmit_ISR);
    // Set the EUSART1 module to the options selected in the user interface.

    /* By setting the Tris control bits for  RXx/DTx  and  TXx/CKx  pins
     * the   EUSART   control   will automatically reconfigure the pin from input to output, as
     * needed*/


    setBaudrate(baudrateIndex);
    
    TRISCbits.TRISC6 = 1; // rx input As Prescribed in Datasheet
    TRISCbits.TRISC7 = 1; //tx output As Prescribed in Datasheet

    RC1STAbits.SPEN = 0x01;
    TX1STAbits.SYNC = 0x00;

    RC1STAbits.CREN = 1;
    RC1STAbits.RX9 = 0;

    TX1STAbits.TXEN = 1;
    TX1STAbits.TX9 = 0;


    // initializing the driver state
    eusart1TxHead = 0;
    eusart1TxTail = 0;
    eusart1TxBufferRemaining = sizeof(eusart1TxBuffer);

    eusart1RxHead = 0;
    eusart1RxTail = 0;
    eusart1RxCount = 0;
    // enable receive interrupt
    PIE1bits.RC1IE = 1;
}

bool
setBaudrate(uint8_t baudrateIndex)
{
    bool ret = true;
    // disable interrupts before changing states
    PIE1bits.RC1IE = 0;
    EUSART1_SetRxInterruptHandler(EUSART1_Receive_ISR);
    PIE1bits.TX1IE = 0;
    EUSART1_SetTxInterruptHandler(EUSART1_Transmit_ISR);
    // Set the EUSART1 module to the options selected in the user interface.

    /* By setting the Tris control bits for  RXx/DTx  and  TXx/CKx  pins
     * the   EUSART   control   will automatically reconfigure the pin from input to output, as
     * needed*/

    switch (baudrateIndex)
    {
            /* AMB2621_BaudRate_9600 */
        case 0:
        {
            SP1BRGH = 0x0000;
            SP1BRGL = 0x67;
        }

            /* AMB2621_BaudRate_19200 */
        case 1:
        {

            SP1BRGH = 0x0000;
            SP1BRGL = 0x33;
        }
            break;
            /* AMB2621_BaudRate_38400*/
        case 2:
        {
            SP1BRGH = 0x0000;
            SP1BRGL = 0x19;
        }
            break;

            /* AMB2621_BaudRate_115200 */
        case 3:
        {
            SP1BRGH = 0x0000;
            SP1BRGL = 0x08;

        }
            break;
        default:
        {
            ret = false;
        }
        break;
    }

    if(ret)
    {
        initUartTimer(baudrateIndex);
    }

    TX1STAbits.BRGH = 0x01;
    BAUD1CONbits.BRG16 = 0x00;
    BAUD1CONbits.SCKP = 0x00;

    // enable receive interrupt
    PIE1bits.RC1IE = 1;
    
    return ret;

}

void
flushRxBuffer()
{
    PIE1bits.RC1IE = 0;
    eusart1RxHead = 0;
    eusart1RxTail = 0;
    eusart1RxCount = 0;
    PIE1bits.RC1IE = 1;

}

uint8_t
EUSART1_is_tx_ready(void)
{
    return eusart1TxBufferRemaining;
}

uint8_t
EUSART1_is_rx_ready(void)
{
    return eusart1RxCount;
}

bool
EUSART1_is_tx_done(void)
{
    return TX1STAbits.TRMT;
}

uint8_t
EUSART1_Read(void)
{
    uint8_t readValue = 0;

    while(0 == eusart1RxCount)
    {
    }

    readValue = eusart1RxBuffer[eusart1RxTail++];
    if (sizeof(eusart1RxBuffer) <= eusart1RxTail)
    {
        eusart1RxTail = 0;
    }
    PIE1bits.RC1IE = 0;
    eusart1RxCount--;
    PIE1bits.RC1IE = 1;

    return readValue;
}

void
EUSART1_Write(uint8_t txData)
{
    while(0 == eusart1TxBufferRemaining)
    {
    }

    if (0 == PIE1bits.TX1IE)
    {
        TX1REG = txData;
    }
    else
    {
        PIE1bits.TX1IE = 0;
        eusart1TxBuffer[eusart1TxHead++] = txData;
        if (sizeof(eusart1TxBuffer) <= eusart1TxHead)
        {
            eusart1TxHead = 0;
        }
        eusart1TxBufferRemaining--;
    }
    PIE1bits.TX1IE = 1;
}

void
EUSART1_Transmit_ISR(void)
{

    // add your EUSART1 interrupt custom code
    if (sizeof(eusart1TxBuffer) > eusart1TxBufferRemaining)
    {
        TX1REG = eusart1TxBuffer[eusart1TxTail++];
        if (sizeof(eusart1TxBuffer) <= eusart1TxTail)
        {
            eusart1TxTail = 0;
        }
        eusart1TxBufferRemaining++;
    }
    else
    {
        PIE1bits.TX1IE = 0;
    }
}

void
EUSART1_Receive_ISR(void)
{

    if (1 == RC1STAbits.OERR)
    {
        // EUSART1 error - restart

        RC1STAbits.CREN = 0;
        RC1STAbits.CREN = 1;
    }

    // buffer overruns are ignored
    eusart1RxBuffer[eusart1RxHead++] = RC1REG;
    if (sizeof(eusart1RxBuffer) <= eusart1RxHead)
    {
        eusart1RxHead = 0;
    }
    eusart1RxCount++;
}

void
EUSART1_SetTxInterruptHandler(void (* interruptHandler)(void))
{
    EUSART1_TxDefaultInterruptHandler = interruptHandler;
}

void
EUSART1_SetRxInterruptHandler(void (* interruptHandler)(void))
{
    EUSART1_RxDefaultInterruptHandler = interruptHandler;
}

static void
initUartTimer(uint8_t baudrateIndex)
{
    /* use timer 4 - timer 2 is already used for PWM!*/

    /* Disabl interrupt */
    PIE3bits.TMR4IE = 0;

    /* Disable Timer*/
    T4CONbits.TMR4ON = 0;

    /*
     * Postscaler and Prescaler are cleared when any timer2 related bit is written, expect for TMR2ON.
     * So configure them last
     */
    
    /* freq of timer = fosc/4 => 4MHz
     * Prescaler: amount of ticks after which timer reg is increased 
     * Prescaler 1: 4000 ticks/ms
     * Prescaler 4: 1000 ticks/ms
     * Prescaler 16: 250 ticks/ms
     * Prescaler 64:  63 ticks/ms
     */
    switch (baudrateIndex)
    {
            /* AMB2621_BaudRate_9600 */
        case 0:
        {
            PR4 = 0xFC; /* Set to 0xFC = 252 which takes 4ms with prescaler 1:64 */

            /* Set Postscaler to 1:11*/
            T4CONbits.T4OUTPS0 = 1;
            T4CONbits.T4OUTPS1 = 1;
            T4CONbits.T4OUTPS2 = 1;
            T4CONbits.T4OUTPS3 = 1;
          
            /* Set Prescaler to 1:64 */
            T4CONbits.T4CKPS0 = 1;
            T4CONbits.T4CKPS1 = 1;
        }

            /* AMB2621_BaudRate_19200 */
        case 1:
        {

            PR4 = 0xBD; /* Set to 0xBD = 189 which takes 3ms with prescaler 1:64 */

            /* Set Postscaler to 1:11*/
            T4CONbits.T4OUTPS0 = 0;
            T4CONbits.T4OUTPS1 = 1;
            T4CONbits.T4OUTPS2 = 0;
            T4CONbits.T4OUTPS3 = 1;
          
            /* Set Prescaler to 1:64 */
            T4CONbits.T4CKPS0 = 1;
            T4CONbits.T4CKPS1 = 1;
        }
            break;
            /* AMB2621_BaudRate_38400*/
        case 2:
        {
            PR4 = 0xFA; /* = 250 which are ticks per ms with prescaler 1:16*/

            /* Set Postscaler to 1:12*/
            T4CONbits.T4OUTPS0 = 1;
            T4CONbits.T4OUTPS1 = 1;
            T4CONbits.T4OUTPS2 = 0;
            T4CONbits.T4OUTPS3 = 1;

            /* Set Prescaler to 1:16 */
            T4CONbits.T4CKPS0 = 0;
            T4CONbits.T4CKPS1 = 1;
        }
            break;

            /* AMB2621_BaudRate_115200 */
        case 3:
        {
            PR4 = 0xFA; /* = 250 which are ticks per ms with prescaler 1:16*/

            /* Set Postscaler to 1:6 */
            T4CONbits.T4OUTPS0 = 1;
            T4CONbits.T4OUTPS1 = 0;
            T4CONbits.T4OUTPS2 = 1;
            T4CONbits.T4OUTPS3 = 0;

            /* Set Prescaler to 1:16 */
            T4CONbits.T4CKPS0 = 0;
            T4CONbits.T4CKPS1 = 1;
        }
            break;
    }
}

void
startUartTimer()
{
    /* Clear Flag */
    PIR3bits.TMR4IF = 0;
    PIE3bits.TMR4IE = 1;
    T4CONbits.TMR4ON = 1;
}

void
stopUartTimer()
{
    PIR3bits.TMR4IF = 0;
    PIE3bits.TMR4IE = 0;
    T4CONbits.TMR4ON = 0;
}

/**
  End of File
 */
